#!/usr/env/bin python
# -*- coding: utf-8 -*-
"""
Bitcoin price form the command line, so sweet
This is just an interface between the CLI and Blockchain.info charts API
"""

from datetime import datetime
import json
import click
import requests
from requests.exceptions import ConnectionError
from tabletext import to_text


class Btcprice(object):
    "BTC price class to interact with the Blockchain API"

    api_url = 'https://api.blockchain.info/charts/market-price'
    options = {}

    def __init__(self):
        "init function"
        self.options['timespan'] = 10
        self.options['format'] = 'json'
        self.options['output'] = 'raw'

    def config(self, output, timespan):
        "setup class configuration"
        self.options['timespan'] = self.__validate_timespan(timespan)
        self.options['output'] = output

    def display(self):
        "display data"
        data = self.call_api()
        if data:
            print('')
            print(data['description'])
            print('')
            print(to_text(self.__get_table(data), header=False, corners='+', hor='-', ver='|'))

    def call_api(self):
        "call api"
        # tha call url
        api_call_url = '%s?format=%s&timespan=%sdays' % (
            self.api_url,
            self.options['format'],
            self.options['timespan']
        )
        if self.options['output'] == 'average':
            api_call_url += '&daysAverageString=7'
        # make the call
        req = requests.get(api_call_url, headers={'Content-type': 'application/json'})
        try:
            return json.loads(req.text)
        except ConnectionError as error:
            print(error)
            return None

    def __get_table(self, data):
        # parse data
        entries = []
        for entry in data['values']:
            entries.append([
                self.__format_timestamp(entry['x']),
                self.__format_float(entry['y'])
            ])
        data_sorted = sorted(entries, key=self.__get_key, reverse=True)
        data_sorted.insert(0, ['Date', 'Price'])
        return data_sorted

    @staticmethod
    def __validate_timespan(timespan):
        if timespan < 5:
            return 5
        elif timespan > 30:
            return 30
        return timespan

    @staticmethod
    def __format_timestamp(timestamp):
        timestamp = datetime.fromtimestamp(int(timestamp))
        return timestamp.strftime('%Y-%m-%d')

    @staticmethod
    def __format_float(number):
        return 'US$ %.2f' % float(number)

    @staticmethod
    def __get_key(item):
        return item[0]


@click.command()
@click.option('--output', '-o', 'output', type=click.Choice(['raw', 'average']), default='raw', help='Set output format: raw, average')
@click.option('--timespan', '-t', 'timespan', type=int, default=10, help='Set data time span: min 5, max 30')
def cli(output, timespan):
    "BTCprice api call"
    btcp = Btcprice()
    btcp.config(output, timespan)
    btcp.display()
